import { isNotNil } from '@alfa-client/tech-shared';
import { Injectable } from '@angular/core';
import { AbstractControl, FormControl, UntypedFormBuilder, UntypedFormGroup } from '@angular/forms';
import { Observable, catchError, of } from 'rxjs';
import {
  Organisationseinheit,
  OrganisationseinheitError,
  OrganisationseinheitErrorType,
} from '../../../user/user.model';
import { UserService } from '../../../user/user.service';
import { getOrganisationseinheitErrorMessage } from '../../../user/user.util';

@Injectable()
export class OrganisationseinheitFormservice {
  public static readonly ORGANISATIONSEINHEIT_NAME_FIELD: string = 'name';
  public static readonly ORGANISATIONSEINHEIT_IDS_FIELD: string = 'organisationseinheit';

  form: UntypedFormGroup;

  source: Organisationseinheit;

  constructor(
    private formBuilder: UntypedFormBuilder,
    private userService: UserService,
  ) {
    this.form = this.formBuilder.group({
      [OrganisationseinheitFormservice.ORGANISATIONSEINHEIT_NAME_FIELD]: new FormControl(''),
      [OrganisationseinheitFormservice.ORGANISATIONSEINHEIT_IDS_FIELD]: new FormControl(''),
    });
  }

  public submit(): Observable<boolean> {
    if (this.validate()) {
      return this.callService().pipe(
        catchError((error: OrganisationseinheitError) => {
          this.handleError(error);
          return of(false);
        }),
      );
    } else {
      return of(false);
    }
  }

  callService(): Observable<boolean> {
    return this.isPatch() ? this.save() : this.create();
  }

  create(): Observable<boolean> {
    return this.userService.createOrganisationseinheit(
      this.getName(),
      this.getOrganisationseinheitIds(),
    );
  }

  save(): Observable<boolean> {
    return this.userService.saveOrganisationseinheit({
      ...this.source,
      name: this.getName(),
      organisationseinheitIds: this.getOrganisationseinheitIds(),
    });
  }

  validate(): boolean {
    let valid: boolean = true;

    if (this.getOrganisationseinheitIds().length == 0) {
      this.setError(OrganisationseinheitFormservice.ORGANISATIONSEINHEIT_IDS_FIELD, {
        errorType: OrganisationseinheitErrorType.ID_MISSING,
        detail: '',
      });
      valid = false;
    }

    return valid;
  }

  private getName(): string {
    return this.getStringFromPotentiallyEmptyField(
      OrganisationseinheitFormservice.ORGANISATIONSEINHEIT_NAME_FIELD,
    );
  }

  private getOrganisationseinheitIds(): string[] {
    return this.splitOrganisationseinheitIds(
      this.form.get(OrganisationseinheitFormservice.ORGANISATIONSEINHEIT_IDS_FIELD).value ?? '',
    );
  }

  private getStringFromPotentiallyEmptyField(fieldName: string): string {
    return this.form.get(fieldName).value ?? '';
  }

  public isPatch(): boolean {
    return isNotNil(this.source);
  }

  handleError(error: OrganisationseinheitError): void {
    if (
      error.errorType === OrganisationseinheitErrorType.NAME_CONFLICT ||
      error.errorType === OrganisationseinheitErrorType.NAME_MISSING
    ) {
      this.setError(OrganisationseinheitFormservice.ORGANISATIONSEINHEIT_NAME_FIELD, error);
    }
  }

  private setError(controlName: string, error: OrganisationseinheitError): void {
    const control: AbstractControl = this.form.get(controlName);
    control.setErrors({
      [error.errorType]: getOrganisationseinheitErrorMessage(error),
    });
  }

  splitOrganisationseinheitIds(organisationseinheitIdsString: string): string[] {
    return organisationseinheitIdsString
      .split(',')
      .map((organisationseinheitId) => organisationseinheitId.trim())
      .filter((organisationseinheitId) => organisationseinheitId.length > 0);
  }

  public patch(organisationseinheit: Organisationseinheit): void {
    this.reset();
    this.source = organisationseinheit;
    this.form.patchValue({
      [OrganisationseinheitFormservice.ORGANISATIONSEINHEIT_NAME_FIELD]: organisationseinheit.name,
      [OrganisationseinheitFormservice.ORGANISATIONSEINHEIT_IDS_FIELD]:
        organisationseinheit.organisationseinheitIds.join(', '),
    });
  }

  public reset(): void {
    this.source = null;
    this.form.reset();
    this.form.markAsUntouched();
  }

  public isInvalid(): boolean {
    return this.form.invalid;
  }
}
