import { ProblemDetail, createStateResource } from '@alfa-client/tech-shared';
import {
  Mock,
  dispatchEventFromFixture,
  existsAsHtmlElement,
  getDebugElementFromFixtureByCss,
  getElementFromFixture,
  mock,
  notExistsAsHtmlElement,
} from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent, ngMocks } from 'ng-mocks';
import { EMPTY, of } from 'rxjs';
import { createInvalidParam, createProblemDetail } from '../../../../../../tech-shared/test/error';
import { singleCold } from '../../../../../../tech-shared/test/marbles';
import { createPostfachResource } from '../../../../../test/postfach/postfach';
import { PrimaryButtonComponent } from '../../../shared/primary-button/primary-button.component';
import { TextFieldComponent } from '../../../shared/text-field/text-field.component';
import { PostfachResource } from '../../postfach.model';
import { PostfachService } from '../../postfach.service';
import { PostfachFormComponent } from './postfach-form.component';
import { PostfachFormService } from './postfach.formservice';

describe('PostfachFormComponent', () => {
  let component: PostfachFormComponent;
  let fixture: ComponentFixture<PostfachFormComponent>;
  let formService: PostfachFormService;

  const postfachService: Mock<PostfachService> = mock(PostfachService);

  const saveButtonSelector: string = getDataTestIdOf('save-button');
  const invalidMessageSpan: string = getDataTestIdOf('invalid-empty-message-span');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        PostfachFormComponent,
        MockComponent(PrimaryButtonComponent),
        MockComponent(TextFieldComponent),
      ],
      imports: [ReactiveFormsModule, FormsModule],
      providers: [
        {
          provide: PostfachService,
          useValue: postfachService,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(PostfachFormComponent);
    component = fixture.componentInstance;
    formService = component.formService;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('set postfachStateResource', () => {
    it('should return resource', () => {
      const updatePostfachResourceFn: jest.SpyInstance = jest.spyOn(
        component,
        'updatePostfachResource',
      );
      const postfachResource: PostfachResource = createPostfachResource();

      component.postfachStateResource = createStateResource(postfachResource);

      expect(updatePostfachResourceFn).toHaveBeenCalledWith(postfachResource);
    });
  });

  describe('updatePostfachResource', () => {
    it('should patch form with input postfach resource', () => {
      const patchFn = jest.spyOn(formService, 'patch');
      const postfachResource: PostfachResource = createPostfachResource();

      component.updatePostfachResource(postfachResource);

      expect(patchFn).toHaveBeenCalledWith(postfachResource.settingBody);
    });
  });

  describe('Absender section', () => {
    const fields: string[][] = [
      [PostfachFormService.NAME_FIELD, 'Name', 'absender-name'],
      [PostfachFormService.ANSCHRIFT_FIELD, 'Anschrift', 'absender-anschrift'],
      [PostfachFormService.DIENST_FIELD, 'Dienst', 'absender-dienst'],
      [PostfachFormService.MANDANT_FIELD, 'Mandant', 'absender-mandant'],
      [
        PostfachFormService.GEMEINDESCHLUESSEL_FIELD,
        'Gemeindeschlüssel',
        'absender-gemeindeschluessel',
      ],
    ];

    test.each(fields)(
      'should bind form for text-field "%s"',
      async (fieldName, text, dataTestId) => {
        const fieldValue = `some text-field ${text}`;
        const absenderGroup = formService.form.get(PostfachFormService.ASBSENDER_GROUP);
        const formControl = absenderGroup.get(fieldName);

        const textFieldComponent = getDebugElementFromFixtureByCss(
          fixture,
          getDataTestIdOf(dataTestId),
        );
        ngMocks.change(textFieldComponent, fieldValue);
        expect(formControl.value).toBe(fieldValue);
      },
    );

    test.each(fields)(
      'should have label for field "%s" with name "%s"',
      (fieldName, text, dataTestId) => {
        const textFieldElement = getElementFromFixture(fixture, getDataTestIdOf(dataTestId));
        expect(textFieldElement.getAttribute('ng-reflect-label')).toBe(text);
      },
    );
  });

  describe('Signatur section', () => {
    const signaturTextarea = getDataTestIdOf('signatur-text');

    it('should have signatur-text', () => {
      existsAsHtmlElement(fixture, signaturTextarea);
    });

    it('should set form-control-name', () => {
      const text = 'some signature text';
      formService.form.get(PostfachFormService.SIGNATUR_FIELD).setValue(text);

      const textareaElement = getElementFromFixture(fixture, signaturTextarea);
      expect(textareaElement.value).toBe(text);
    });
  });

  describe('save button', () => {
    it('should call submit on click', () => {
      const submitFn: jest.SpyInstance = jest.spyOn(formService, 'submit');
      postfachService.save.mockReturnValue(EMPTY);

      dispatchEventFromFixture(fixture, saveButtonSelector, 'clickEmitter');

      expect(submitFn).toHaveBeenCalled();
    });

    it.each([true, false])('should use submit progress "%s"', (progress) => {
      component.submitInProgress$ = of(progress);

      fixture.detectChanges();

      const saveButtonComponent: PrimaryButtonComponent = getDebugElementFromFixtureByCss(
        fixture,
        saveButtonSelector,
      ).componentInstance;
      expect(saveButtonComponent.submitInProgress).toBe(progress);
    });
  });

  describe('invalid message', () => {
    it('should show if form invalidEmpty', () => {
      formService.setErrorByProblemDetail(createProblemDetailForAbsenderName());

      fixture.detectChanges();

      existsAsHtmlElement(fixture, invalidMessageSpan);
    });

    function createProblemDetailForAbsenderName(): ProblemDetail {
      return {
        ...createProblemDetail(),
        invalidParams: [{ ...createInvalidParam(), name: 'settingBody.absender.name' }],
      };
    }

    it('should not show if form valid', () => {
      expect(formService.form.invalid).toBeFalsy();

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, invalidMessageSpan);
    });
  });

  describe('submit', () => {
    it('should assign submit in progress', () => {
      formService.submitWithProgress = jest.fn().mockReturnValue(singleCold(true));

      component.submit();

      expect(component.submitInProgress$).toBeObservable(singleCold(true));
    });
  });
});
