import {
  createEmptyStateResource,
  createStateResource,
  StateResource,
} from '@alfa-client/tech-shared';
import { mock, Mock, useFromMock } from '@alfa-client/test-utils';
import { FormBuilder } from '@angular/forms';
import { cold } from 'jest-marbles';
import { Observable, of } from 'rxjs';
import { createPostfach, createPostfachResource } from '../../../../../test/postfach/postfach';
import { Postfach, PostfachResource } from '../../postfach.model';
import { PostfachService } from '../../postfach.service';
import { PostfachFormService } from './postfach.formservice';

describe('PostfachFormService', () => {
  let formService: PostfachFormService;
  let postfachService: Mock<PostfachService>;
  const formBuilder: FormBuilder = new FormBuilder();

  beforeEach(() => {
    postfachService = mock(PostfachService);

    formService = new PostfachFormService(formBuilder, useFromMock(postfachService));
  });

  it('should create', () => {
    expect(formService).toBeTruthy();
  });
  describe('submit', () => {
    const postfach: Postfach = createPostfach();

    beforeEach(() => {
      const stateResource: StateResource<PostfachResource> =
        createStateResource(createPostfachResource());
      postfachService.save.mockReturnValue(of(stateResource));
      postfachService.get.mockReturnValue(of(stateResource));
      formService.form.setValue({
        [PostfachFormService.ASBSENDER_GROUP]: {
          [PostfachFormService.NAME_FIELD]: postfach.absender.name,
          [PostfachFormService.ANSCHRIFT_FIELD]: postfach.absender.anschrift,
          [PostfachFormService.DIENST_FIELD]: postfach.absender.dienst,
          [PostfachFormService.MANDANT_FIELD]: postfach.absender.mandant,
          [PostfachFormService.GEMEINDESCHLUESSEL_FIELD]: postfach.absender.gemeindeschluessel,
        },
        [PostfachFormService.SIGNATUR_FIELD]: postfach.signatur,
      });
    });

    it('should call save on submit', () => {
      formService.submit();

      expect(postfachService.save).toHaveBeenCalledWith(postfach);
    });

    describe('with empty or null absender values', () => {
      const formValueWithAbsender = {
        [PostfachFormService.ASBSENDER_GROUP]: {
          [PostfachFormService.NAME_FIELD]: '',
          [PostfachFormService.ANSCHRIFT_FIELD]: null,
          [PostfachFormService.DIENST_FIELD]: null,
          [PostfachFormService.MANDANT_FIELD]: '',
          [PostfachFormService.GEMEINDESCHLUESSEL_FIELD]: '',
        },
        [PostfachFormService.SIGNATUR_FIELD]: postfach.signatur,
      };

      beforeEach(() => {
        formService.form.setValue(formValueWithAbsender);
        formService.source = { signatur: postfach.signatur };
      });
      it('should call save without absender', () => {
        formService.submit();

        expect(postfachService.save).toHaveBeenCalledWith({ signatur: postfach.signatur });
      });

      it('should call save with absender if any present', () => {
        formValueWithAbsender[PostfachFormService.ASBSENDER_GROUP][PostfachFormService.NAME_FIELD] =
          'something';
        formService.form.setValue(formValueWithAbsender);

        formService.submit();

        expect(postfachService.save).toHaveBeenCalledWith(formValueWithAbsender);
      });

      it('should call save with absender if valid source', () => {
        formService.source = postfach;

        formService.submit();

        expect(postfachService.save).toHaveBeenCalledWith(formValueWithAbsender);
      });
    });
  });

  describe('submit with progress', () => {
    it('should assign submit in progress to state resource loading', () => {
      formService.submit = jest.fn().mockReturnValue(
        cold('ab', {
          a: createEmptyStateResource(true),
          b: createEmptyStateResource(false),
        }),
      );

      const progressObservable: Observable<boolean> = formService.submitWithProgress();

      expect(progressObservable).toBeObservable(cold('ab', { a: true, b: false }));
    });
  });
});
