import {
  OrganisationsEinheitResource,
  OrganisationsEinheitService,
} from '@alfa-client/collaboration-shared';
import {
  Mock,
  dispatchEventFromFixture,
  existsAsHtmlElement,
  getMockComponent,
  mock,
} from '@alfa-client/test-utils';
import { OzgcloudDialogService } from '@alfa-client/ui';
import { DialogConfig } from '@angular/cdk/dialog';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormControl } from '@angular/forms';
import { getUrl } from '@ngxp/rest';
import { ButtonComponent, OfficeIconComponent } from '@ods/system';
import { createOrganisationsEinheitResource } from 'libs/collaboration-shared/test/organisations-einheit';
import { SearchIconComponent } from 'libs/design-system/src/lib/icons/search-icon/search-icon.component';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { SearchOrganisationsEinheitContainerComponent } from '../../../search-organisations-einheit-container/search-organisations-einheit-container.component';
import { OrganisationsEinheitContainerComponent } from './organisations-einheit-container.component';
import { OrganisationsEinheitComponent } from './organisations-einheit/organisations-einheit.component';

describe('OrganisationsEinheitContainerComponent', () => {
  let component: OrganisationsEinheitContainerComponent;
  let fixture: ComponentFixture<OrganisationsEinheitContainerComponent>;

  const searchOrganisationsEinheitButton: string = getDataTestIdOf(
    'organisations-einheit-search-button',
  );
  const organisationsEinheitComp: string = getDataTestIdOf(
    'organisations-einheit-in-collaboration',
  );

  const organisationsEinheitResource: OrganisationsEinheitResource =
    createOrganisationsEinheitResource();

  let dialogService: Mock<OzgcloudDialogService>;
  let service: Mock<OrganisationsEinheitService>;

  beforeEach(async () => {
    dialogService = mock(OzgcloudDialogService);
    service = {
      ...mock(OrganisationsEinheitService),
      getSelectedResult: jest.fn().mockReturnValue(of(createOrganisationsEinheitResource())),
    };
    TestBed.overrideComponent(OrganisationsEinheitContainerComponent, {
      set: {
        providers: [
          {
            provide: OrganisationsEinheitService,
            useValue: service,
          },
        ],
      },
    });
    await TestBed.configureTestingModule({
      declarations: [
        OrganisationsEinheitContainerComponent,
        MockComponent(SearchIconComponent),
        MockComponent(OfficeIconComponent),
        MockComponent(ButtonComponent),
        MockComponent(OrganisationsEinheitComponent),
      ],
      providers: [
        {
          provide: OzgcloudDialogService,
          useValue: dialogService,
        },
        {
          provide: OrganisationsEinheitService,
          useValue: service,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(OrganisationsEinheitContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInt', () => {
    beforeEach(() => {
      component.getSelectedResult = jest.fn().mockReturnValue(of(organisationsEinheitResource));
    });

    it('should call getSelectedResult', () => {
      component.getSelectedResult = jest.fn();

      component.ngOnInit();

      expect(component.getSelectedResult).toHaveBeenCalled();
    });

    it('should set organisationsEinheitResource', (done) => {
      component.ngOnInit();

      component.organisationsEinheitResource$.subscribe(
        (organisationsEinheitResource: OrganisationsEinheitResource) => {
          expect(organisationsEinheitResource).toBe(organisationsEinheitResource);
          done();
        },
      );
    });
  });

  describe('get selected result', () => {
    const organisationsEinheitResource: OrganisationsEinheitResource =
      createOrganisationsEinheitResource();

    beforeEach(() => {
      service.getSelectedResult.mockReturnValue(of(organisationsEinheitResource));
    });

    it('should call service', () => {
      component.getSelectedResult().subscribe();

      expect(service.getSelectedResult).toHaveBeenCalled();
    });

    it('should call handleResult', () => {
      component.handleResult = jest.fn();

      component.getSelectedResult().subscribe();

      expect(component.handleResult).toHaveBeenCalledWith(organisationsEinheitResource);
    });

    it('should return value', (done) => {
      component.handleResult = jest.fn();

      component.getSelectedResult().subscribe((result) => {
        expect(result).toBe(organisationsEinheitResource);
        done();
      });
    });
  });

  describe('handle result', () => {
    beforeEach(() => {
      component.fieldControl = new FormControl();
    });

    it('should patch fieldControl with resource uri', () => {
      const fieldControlPatchSpy: jest.SpyInstance = (component.fieldControl.patchValue =
        jest.fn());

      component.handleResult(organisationsEinheitResource);

      expect(fieldControlPatchSpy).toHaveBeenCalledWith(getUrl(organisationsEinheitResource));
    });

    it('should not patch fieldControl if organisationsEinheit resource is null', () => {
      const fieldControlPatchSpy: jest.SpyInstance = (component.fieldControl.patchValue =
        jest.fn());

      component.handleResult(null);

      expect(fieldControlPatchSpy).not.toHaveBeenCalled();
    });
  });

  describe('search zustaendige stelle button', () => {
    beforeEach(() => {
      component.organisationsEinheitResource$ = of(undefined);
    });

    it('should be visible on missing organisationsEinheit', () => {
      fixture.detectChanges();

      existsAsHtmlElement(fixture, searchOrganisationsEinheitButton);
    });
    it('should call openSearchDialog', () => {
      component.openSearchDialog = jest.fn();

      fixture.detectChanges();
      dispatchEventFromFixture(fixture, searchOrganisationsEinheitButton, 'clickEmitter');

      expect(component.openSearchDialog).toHaveBeenCalled();
    });
  });

  describe('organisationsEinheit component', () => {
    beforeEach(() => {
      component.organisationsEinheitResource$ = of(organisationsEinheitResource);
    });

    it('should be visible on existing organisationsEinheit', () => {
      fixture.detectChanges();

      existsAsHtmlElement(fixture, organisationsEinheitComp);
    });

    it('should be called with resource', () => {
      fixture.detectChanges();

      const comp: OrganisationsEinheitComponent = getMockComponent<OrganisationsEinheitComponent>(
        fixture,
        OrganisationsEinheitComponent,
      );
      expect(comp.organisationsEinheitResource).toBe(organisationsEinheitResource);
    });
  });

  describe('open search dialog', () => {
    it('should call dialog service', () => {
      const DIALOG_CONFIG: DialogConfig = {
        backdropClass: ['backdrop-blur-1', 'bg-greybackdrop'],
      };

      component.openSearchDialog();

      expect(dialogService.openInCallingComponentContext).toHaveBeenCalledWith(
        SearchOrganisationsEinheitContainerComponent,
        component.viewContainerRef,
        null,
        DIALOG_CONFIG,
      );
    });
  });
});
