import { HttpErrorResponse } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { CommandResource } from '@alfa-client/command-shared';
import { NavigationService } from '@alfa-client/navigation-shared';
import { StateResource, isNotFound } from '@alfa-client/tech-shared';
import {
  AdditionalActions,
  VorgangService,
  VorgangWithEingangResource,
} from '@alfa-client/vorgang-shared';
import { Observable, map, tap, withLatestFrom } from 'rxjs';
import { LoeschAnforderungFacade } from './+state/loesch-anforderung.facade';
import { LoeschAnforderungResource } from './loesch-anforderung.model';
import {
  createLoeschAnforderungZuruecknehmenCommand,
  createVorgangLoeschenCommand,
  createVorgangZumLoeschenMarkierenCommand,
} from './loesch-anforderung.util';

import * as CommandUtil from '../../../command-shared/src/lib/command.util';
import * as LoeschAnforderungActions from './+state/loesch-anforderung.actions';
@Injectable()
export class LoeschAnforderungService {
  constructor(
    private facade: LoeschAnforderungFacade,
    private vorgangService: VorgangService,
    private navigationService: NavigationService,
  ) {}

  public getLoeschenAnfordernCommand(): Observable<StateResource<CommandResource>> {
    return this.facade.getLoeschenAnfordernCommand();
  }

  public loeschenAnfordern(vorgang: VorgangWithEingangResource): void {
    this.facade.loeschenAnfordern(vorgang, createVorgangZumLoeschenMarkierenCommand());
  }

  public getLoeschAnforderung(): Observable<StateResource<LoeschAnforderungResource>> {
    return this.facade.getLoeschAnforderung();
  }

  public getEndgueltigLoeschenCommand(): Observable<StateResource<CommandResource>> {
    return this.facade.getEndgueltigLoeschenCommand().pipe(
      withLatestFrom(this.vorgangService.getBackButtonUrl()),
      tap(([commandStatResource, backButtonUrl]) =>
        this.handleEndgueltigLoeschenCommandChanges(commandStatResource.resource, backButtonUrl),
      ),
      map(([commandStatResource]) => commandStatResource),
    );
  }

  handleEndgueltigLoeschenCommandChanges(
    commandResource: CommandResource,
    backButtonUrl: string,
  ): void {
    if (CommandUtil.isSuccessfulDone(commandResource)) {
      this.navigationService.navigate(backButtonUrl);
    }
  }

  public endgueltigLoeschen(loeschAnforderung: LoeschAnforderungResource): void {
    this.facade.endgueltigLoeschen(loeschAnforderung, createVorgangLoeschenCommand());
  }

  public getLoschAnforderungZuruecknehmenCommand(): Observable<StateResource<CommandResource>> {
    return this.facade.getLoeschAnforderungZuruecknehmenCommand().pipe(
      withLatestFrom(this.vorgangService.getBackButtonUrl()),
      tap(([commandStatResource, overviewUrl]) =>
        this.handleLoeschAnforderungZuruecknehmenCommandChanges(commandStatResource, overviewUrl),
      ),
      map(([commandStatResource]) => commandStatResource),
    );
  }

  handleLoeschAnforderungZuruecknehmenCommandChanges(
    commandStateResource: StateResource<CommandResource>,
    overviewUrl: string,
  ): void {
    if (isNotFound((<HttpErrorResponse>(<unknown>commandStateResource?.error))?.status)) {
      this.vorgangService.reloadCurrentVorgangWithAddtionalActions(
        this.createAdditionalActions(overviewUrl),
      );
    }
  }

  createAdditionalActions(overviewUrl: string): AdditionalActions {
    const additionalSuccessAction =
      LoeschAnforderungActions.loeschAnforderungZuruecknehmenFailedAndReloadVorgangSuccess;
    const additionalFailureAction = (error: HttpErrorResponse) =>
      LoeschAnforderungActions.loeschAnforderungZuruecknehmenFailedAndReloadVorgangFailure({
        error,
        overviewUrl,
      });
    return { additionalSuccessAction, additionalFailureAction };
  }

  public zuruecknehmen(loeschAnforderung: LoeschAnforderungResource): void {
    this.facade.loeschAnforderungZuruecknehmen(
      loeschAnforderung,
      createLoeschAnforderungZuruecknehmenCommand(),
    );
  }
}
