/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  BinaryFileListResource,
  LoadBinaryFileListSuccessProps,
} from '@alfa-client/binary-file-shared';
import {
  ApiError,
  ApiErrorAction,
  EMPTY_ACTION,
  createEmptyStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { TypedAction } from '@ngrx/store/src/models';
import { createBinaryFileListResource } from 'libs/binary-file-shared/test/binary-file';
import { createPostfachMailResource } from 'libs/postfach-shared/test/postfach';
import { createApiError } from 'libs/tech-shared/test/error';
import { PostfachMailResource } from '../postfach.model';
import { PostfachNachrichtProps } from './postfach.actions.model';
import { PostfachState, initialPostfachState, postfachReducer } from './postfach.reducer';

import * as PostfachActions from './postfach.actions';

describe('Postfach Reducer', () => {
  describe('unknown action', () => {
    it('should return current state', () => {
      const state: PostfachState = postfachReducer(initialPostfachState, EMPTY_ACTION);

      expect(state).toBe(initialPostfachState);
    });
  });

  describe('isDownloadPdfInProgress', () => {
    describe('on "startDownloadAsPdf" action', () => {
      it('should set to true', () => {
        const action: TypedAction<string> = PostfachActions.startDownloadAsPdf();

        const state: PostfachState = postfachReducer(initialPostfachState, action);

        expect(state.isDownloadPdfInProgress).toBeTruthy();
      });
    });

    describe('on "downloadAsPdfSuccess" action', () => {
      it('should set to false', () => {
        const action: TypedAction<string> = PostfachActions.downloadAsPdfSuccess();

        const state: PostfachState = postfachReducer(
          { ...initialPostfachState, isDownloadPdfInProgress: true },
          action,
        );

        expect(state.isDownloadPdfInProgress).toBeFalsy();
      });
    });

    describe('on "downloadAsPdfFailed" action', () => {
      it('should set to false', () => {
        const action: ApiErrorAction & TypedAction<string> = PostfachActions.downloadAsPdfFailed({
          apiError: createApiError(),
        });

        const state: PostfachState = postfachReducer(
          { ...initialPostfachState, isDownloadPdfInProgress: true },
          action,
        );

        expect(state.isDownloadPdfInProgress).toBeFalsy();
      });
    });
  });

  describe('attachmentList', () => {
    describe('on "loadAttachmentList" action', () => {
      it('should set loading', () => {
        const postfachNachricht: PostfachMailResource = createPostfachMailResource();
        const action: PostfachNachrichtProps & TypedAction<string> =
          PostfachActions.loadAttachmentList({ postfachNachricht });

        const state: PostfachState = postfachReducer(initialPostfachState, action);

        expect(state.attachmentList.loading).toBeTruthy();
      });
    });

    describe('on "loadAttachmentListSuccess" action', () => {
      it('should create stateResource', () => {
        const binaryFileList: BinaryFileListResource = createBinaryFileListResource();
        const action: LoadBinaryFileListSuccessProps & TypedAction<string> =
          PostfachActions.loadAttachmentListSuccess({ binaryFileList });

        const state: PostfachState = postfachReducer(initialPostfachState, action);

        expect(state.attachmentList.resource).toBe(binaryFileList);
      });
    });

    describe('on "loadAttachmentListFailure" action', () => {
      it('should create error stateResource', () => {
        const apiError: ApiError = createApiError();
        const action: ApiErrorAction & TypedAction<string> =
          PostfachActions.loadAttachmentListFailure({ apiError });

        const state: PostfachState = postfachReducer(initialPostfachState, action);

        expect(state.attachmentList.error).toBe(apiError);
      });
    });

    describe('on "clearAttachmentList" action', () => {
      it('should create empty stateResource', () => {
        const action: TypedAction<string> = PostfachActions.clearAttachmentList();

        const state: PostfachState = postfachReducer(
          {
            ...initialPostfachState,
            attachmentList: createStateResource(createBinaryFileListResource()),
          },
          action,
        );

        expect(state.attachmentList).toEqual(createEmptyStateResource());
      });
    });
  });
});
