import {
  COMMAND_ERROR_MESSAGES,
  CommandResource,
  hasCommandError,
  isSuccessfulDone,
} from '@alfa-client/command-shared';
import {
  createEmptyStateResource,
  hasContent,
  isClipboardReadSupported,
  StateResource,
} from '@alfa-client/tech-shared';
import { SnackBarService } from '@alfa-client/ui';
import { VorgangService, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { Component, Inject, OnInit } from '@angular/core';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import { map, Observable, of, startWith, tap } from 'rxjs';
import { AktenzeichenEditDialogData } from './aktenzeichen-edit-dialog.data';
import { AktenzeichenEditDialogMessages } from './aktenzeichen-edit-dialog.message';
import { UntypedFormGroup } from '@angular/forms';
import { AktenzeichenEditDialogFormservice } from './aktenzeichen-edit-dialog.formservice';

@Component({
  selector: 'alfa-aktenzeichen-edit-dialog',
  templateUrl: './aktenzeichen-edit-dialog.component.html',
  styleUrls: ['./aktenzeichen-edit-dialog.component.scss'],
  providers: [AktenzeichenEditDialogFormservice],
})
export class AktenzeichenEditDialogComponent implements OnInit {
  vorgang: VorgangWithEingangResource;

  setAktenzeichenPending$: Observable<StateResource<CommandResource>> = of(
    createEmptyStateResource<CommandResource>(),
  );
  hasAktenzeichen$: Observable<boolean>;
  isPasteSupported = false;

  protected readonly AktenzeichenEditDialogFormservice = AktenzeichenEditDialogFormservice;

  constructor(
    private readonly formService: AktenzeichenEditDialogFormservice,
    private readonly snackBarService: SnackBarService,
    private readonly vorgangService: VorgangService,
    public dialogRef: MatDialogRef<AktenzeichenEditDialogComponent>,
    @Inject(MAT_DIALOG_DATA) public data: AktenzeichenEditDialogData,
  ) {
    this.vorgang = data.vorgang;
  }

  ngOnInit(): void {
    this.isPasteSupported = isClipboardReadSupported();
    this.formService.setAktenzeichen(this.vorgang.aktenzeichen);
    this.hasAktenzeichen$ = this.formService
      .valueChanges()
      .pipe(startWith(this.vorgang.aktenzeichen))
      .pipe(map(hasContent));
  }

  public onCancel(): void {
    this.dialogRef.close();
  }

  public onSubmit(): void {
    this.formService.setVorgangWithEingangResource(this.vorgang);
    this.setAktenzeichenPending$ = this.formService
      .submit()
      .pipe(startWith(createEmptyStateResource<CommandResource>(true)))
      .pipe(
        tap((commandStateResource: StateResource<CommandResource>) =>
          this.onResponse(commandStateResource.resource),
        ),
      );
  }

  onResponse(commandResource: CommandResource) {
    if (commandResource && hasCommandError(commandResource)) {
      this.showSnackbar(commandResource);
    } else if (isSuccessfulDone(commandResource)) {
      this.vorgangService.reloadCurrentVorgang();
      this.dialogRef.close();
    }
  }

  showSnackbar(commandResource: CommandResource): void {
    if (hasCommandError(commandResource)) {
      this.snackBarService.showError(this.getErrorMessage(commandResource));
    }
  }

  getErrorMessage(commandResource: CommandResource): string {
    return (
      COMMAND_ERROR_MESSAGES[commandResource.errorMessage] ||
      AktenzeichenEditDialogMessages.SET_FAILED
    );
  }

  paste(clipboardContent: string) {
    this.formService.setAktenzeichen(clipboardContent);
  }

  public get form(): UntypedFormGroup {
    return this.formService.form;
  }
}
