/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.errorhandling;

import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.UUID;
import java.util.function.Predicate;

import org.apache.commons.lang3.StringUtils;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;

import de.ozgcloud.alfa.common.GrpcUtil;
import de.ozgcloud.common.errorhandling.ExceptionUtil;
import de.ozgcloud.common.errorhandling.FunctionalErrorCode;
import io.grpc.Metadata;
import io.grpc.Status;
import io.grpc.StatusRuntimeException;
import lombok.extern.log4j.Log4j2;

@ControllerAdvice
@Log4j2
@Order(97)
public class GrpcExceptionController {

	static final FunctionalErrorCode GRPC_NOT_FOUND_ERROR_CODE_KEY = () -> ExceptionController.RESOURCE_NOT_FOUNT_MESSAGE_CODE;
	static final FunctionalErrorCode GRPC_INTERNAL_ERROR_CODE_KEY = () -> ExceptionController.RUNTIME_MESSAGE_CODE;
	static final FunctionalErrorCode GRPC_PERMISSION_DENIED_CODE_KEY = () -> ExceptionController.ACCESS_DENIED_MESSAGE_CODE;

	static final String KEY_ERROR_CODE = "ERROR_CODE";
	static final String KEY_EXCEPTION_ID = "EXCEPTION_ID";

	private final Predicate<String> ignoreKey = key -> !isErrorCode(key) && !isExceptionId(key);

	private boolean isErrorCode(String metadataKey) {
		return isParamKeyEquals(metadataKey, KEY_ERROR_CODE);
	}

	private boolean isExceptionId(String metadataKey) {
		return isParamKeyEquals(metadataKey, KEY_EXCEPTION_ID);
	}

	private boolean isParamKeyEquals(String metaDataKey, String key) {
		return GrpcUtil.keyOfString(metaDataKey).equals(GrpcUtil.keyOfString(key));
	}

	@ExceptionHandler(StatusRuntimeException.class)
	public ResponseEntity<ApiError> handleStatusRuntimeException(StatusRuntimeException e) {
		if (isNotFoundStatusCode(e)) {
			return buildNotFoundErrorResponse(e);
		} else if (isInternalStatusCode(e)) {
			return buildInternalErrorResponse(e);
		} else if (isPermissionDeniedCode(e)) {
			return buildAccessDeniedErrorResponse(e);
		}

		return buildInternalUnavailableErrorResponse(e);
	}

	private boolean isNotFoundStatusCode(StatusRuntimeException e) {
		return e.getStatus().getCode() == Status.NOT_FOUND.getCode();
	}

	private ResponseEntity<ApiError> buildNotFoundErrorResponse(StatusRuntimeException e) {
		LOG.error("Grpc not found exception: {}", e.getMessage());

		Issue.IssueBuilder issueBuilder = Issue.builder().message(e.getMessage()).messageCode(GRPC_NOT_FOUND_ERROR_CODE_KEY.getErrorCode());
		addExceptionId(issueBuilder, e);

		return new ResponseEntity<>(buildApiErrorWithIssue(issueBuilder.build()), HttpStatus.NOT_FOUND);
	}

	private ResponseEntity<ApiError> buildInternalUnavailableErrorResponse(StatusRuntimeException e) {
		if (hasExceptionId(e)) {
			return new ResponseEntity<>(buildInternalUnavailableApiError(e, getExceptionId(e)), HttpStatus.SERVICE_UNAVAILABLE);
		}

		var exceptionId = createExceptionId();
		var messageWithExceptionId = ExceptionUtil.formatMessageWithExceptionId(e.getMessage(), exceptionId);
		LOG.error("Grpc service unavailable: {}", messageWithExceptionId);

		return new ResponseEntity<>(buildInternalUnavailableApiError(e, exceptionId), HttpStatus.SERVICE_UNAVAILABLE);
	}

	String createExceptionId() {
		return UUID.randomUUID().toString();
	}

	private ApiError buildInternalUnavailableApiError(StatusRuntimeException e, String exceptionId) {
		return ApiError.builder().issue(Issue.builder().message(e.getMessage()).messageCode(getMessageCode(e)).exceptionId(exceptionId).build())
				.build();
	}

	private ResponseEntity<ApiError> buildInternalErrorResponse(StatusRuntimeException e) {
		LOG.error("Grpc internal server error: " + e.getMessage(), e);

		return new ResponseEntity<>(buildInternalApiError(e), HttpStatus.INTERNAL_SERVER_ERROR);
	}

	private boolean isInternalStatusCode(StatusRuntimeException e) {
		return e.getStatus().getCode() == Status.INTERNAL.getCode();
	}

	private ApiError buildInternalApiError(StatusRuntimeException e) {
		var issueBuilder = Issue.builder().message(e.getMessage()).messageCode(getMessageCode(e));

		addParameter(issueBuilder, e);
		addExceptionId(issueBuilder, e);

		return buildApiErrorWithIssue(issueBuilder.build());
	}

	String getMessageCode(StatusRuntimeException e) {
		if (hasErrorCode(e)) {
			return getErrorCode(e);
		}
		if (hasServiceCode(e)) {
			return getServiceCode(e);
		}
		return GRPC_INTERNAL_ERROR_CODE_KEY.getErrorCode();
	}

	private boolean hasServiceCode(StatusRuntimeException e) {
		return Optional.ofNullable(e.getTrailers()).map(metadata -> metadata.get(GrpcUtil.keyOfString(GrpcUtil.SERVICE_KEY))).isPresent();
	}

	private String getServiceCode(StatusRuntimeException e) {
		return getParameter(e, GrpcUtil.SERVICE_KEY);
	}

	private boolean hasErrorCode(StatusRuntimeException e) {
		return hasParameter(e) && StringUtils.isNotEmpty(getErrorCode(e));
	}

	private ApiError buildApiErrorWithIssue(Issue issue) {
		return ApiError.builder().issue(issue).build();
	}

	private boolean isPermissionDeniedCode(StatusRuntimeException e) {
		return e.getStatus().getCode() == Status.PERMISSION_DENIED.getCode();
	}

	private ResponseEntity<ApiError> buildAccessDeniedErrorResponse(StatusRuntimeException e) {
		var exceptionId = createExceptionId();
		var messageWithExceptionId = ExceptionUtil.formatMessageWithExceptionId(e.getMessage(), exceptionId);
		LOG.error("Grpc permission denied error: " + messageWithExceptionId);

		return new ResponseEntity<>(buildAccessDeniedApiError(e, exceptionId), HttpStatus.FORBIDDEN);
	}

	private ApiError buildAccessDeniedApiError(StatusRuntimeException e, String exceptionId) {
		return ApiError.builder()
				.issue(Issue.builder().message(e.getMessage()).messageCode(GRPC_PERMISSION_DENIED_CODE_KEY.getErrorCode()).exceptionId(exceptionId)
						.build())
				.build();
	}

	void addParameter(Issue.IssueBuilder builder, StatusRuntimeException e) {
		if (hasParameter(e)) {
			builder.parameters(mapToIssueParams(e.getTrailers()));
		}
	}

	List<IssueParam> mapToIssueParams(Metadata metaData) {
		return metaData.keys().stream()
				.filter(ignoreKey)
				.map(key -> buildIssueParam(metaData, key))
				.toList();
	}

	private IssueParam buildIssueParam(Metadata metaData, String key) {
		return IssueParam.builder().name(key).value(metaData.get(GrpcUtil.keyOfString(key))).build();
	}

	void addExceptionId(Issue.IssueBuilder builder, StatusRuntimeException e) {
		if (hasExceptionId(e)) {
			builder.exceptionId(getExceptionId(e));
		}
	}

	private boolean hasExceptionId(StatusRuntimeException e) {
		return hasParameter(e) && StringUtils.isNotEmpty(getExceptionId(e));
	}

	private boolean hasParameter(StatusRuntimeException e) {
		return Objects.nonNull(e.getTrailers());
	}

	private String getExceptionId(StatusRuntimeException e) {
		return getParameter(e, KEY_EXCEPTION_ID);
	}

	private String getErrorCode(StatusRuntimeException e) {
		return getParameter(e, KEY_ERROR_CODE);
	}

	private String getParameter(StatusRuntimeException e, String metadataKey) {
		return e.getTrailers().get(GrpcUtil.keyOfString(metadataKey));
	}
}