/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.RepresentationModel;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PostAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import de.ozgcloud.alfa.common.clientattribute.ClientAttributeService;
import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.statistic.StatisticController;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import jakarta.servlet.http.HttpServletResponse;

@RestController
@RequestMapping(VorgangController.PATH)
public class VorgangController {

	static final String PATH = "/api/vorgangs"; // NOSONAR
	static final String PARAM_SEARCH = "searchBy";
	static final String PARAM_PAGE = "page";
	static final String PARAM_LIMIT = "limit";
	static final String PARAM_ASSIGNED_TO = "assignedTo";
	static final String PARAM_STATUS = "status";
	static final String PARAM_NEXT_WIEDERVORLAGE_FRIST = "nextFrist";
	static final String PARAM_NACHRICHTEN = "nachrichten";

	public static final String PARAM_NEXT_WIEDERVORLAGE_FRIST_EXISTS = "exists";
	public static final String PARAM_NACHRICHTEN_UNGELESENE = "ungelesene";

	@Autowired
	private VorgangService vorgangService;
	@Autowired
	private VorgangModelAssembler modelAssembler;

	@Autowired
	private ClientAttributeService clientAttributeService;

	@Autowired
	private StatisticController statisticController;

	@GetMapping(params = { PARAM_PAGE, PARAM_LIMIT })
	public RepresentationModel<EntityModel<EnhancedVorgang>> getAll(@RequestParam int page, @RequestParam Integer limit) {
		var requestCriteria = FindVorgaengeHeaderRequestCriteria.builder().page(page).limit(limit).build();

		return handleGetAllRequest(requestCriteria);
	}

	@GetMapping(params = { PARAM_PAGE, PARAM_LIMIT, PARAM_STATUS })
	public RepresentationModel<EntityModel<EnhancedVorgang>> getAllByStatus(@RequestParam int page, @RequestParam Integer limit,
			@RequestParam VorgangStatus status) {
		var requestCriteria = FindVorgaengeHeaderRequestCriteria.builder().page(page).limit(limit)
				.filterCriteria(FilterCriteria.builder().status(Optional.of(status)).build()).build();

		return handleGetAllRequest(requestCriteria);
	}

	@GetMapping(params = { PARAM_PAGE, PARAM_LIMIT, PARAM_SEARCH })
	public RepresentationModel<EntityModel<EnhancedVorgang>> getAllBySearchBy(@RequestParam int page, @RequestParam Integer limit,
			@RequestParam String searchBy) {
		var requestCriteria = FindVorgaengeHeaderRequestCriteria.builder().page(page).limit(limit).searchBy(Optional.of(searchBy)).build();

		return handleGetAllRequest(requestCriteria);
	}

	@GetMapping(params = { PARAM_PAGE, PARAM_LIMIT, PARAM_ASSIGNED_TO })
	public RepresentationModel<EntityModel<EnhancedVorgang>> getAllByAssignedTo(@RequestParam int page, @RequestParam Integer limit,
			@RequestParam UserId assignedTo) {
		var requestCriteria = FindVorgaengeHeaderRequestCriteria.builder().page(page).limit(limit)
				.filterCriteria(FilterCriteria.builder().filterByAssignedTo(true).assignedTo(asOptional(assignedTo)).build()).build();

		return handleGetAllRequest(requestCriteria);
	}

	@GetMapping(params = { PARAM_PAGE, PARAM_LIMIT, PARAM_ASSIGNED_TO, PARAM_SEARCH })
	public RepresentationModel<EntityModel<EnhancedVorgang>> getAllByAssignedToAndSearchBy(@RequestParam int page, @RequestParam Integer limit,
			@RequestParam UserId assignedTo, @RequestParam String searchBy) {
		var requestCriteria = FindVorgaengeHeaderRequestCriteria.builder().page(page).limit(limit)
				.filterCriteria(FilterCriteria.builder().filterByAssignedTo(true).assignedTo(asOptional(assignedTo)).build())
				.searchBy(Optional.of(searchBy))
				.build();

		return handleGetAllRequest(requestCriteria);
	}

	@GetMapping(params = { PARAM_PAGE, PARAM_LIMIT, PARAM_ASSIGNED_TO, PARAM_STATUS })
	public RepresentationModel<EntityModel<EnhancedVorgang>> getAllByAssignedToAndStatus(@RequestParam int page, @RequestParam Integer limit,
			@RequestParam UserId assignedTo, @RequestParam VorgangStatus status) {
		var requestCriteria = FindVorgaengeHeaderRequestCriteria.builder().page(page).limit(limit)
				.filterCriteria(FilterCriteria.builder()
						.status(Optional.of(status))
						.filterByAssignedTo(true).assignedTo(asOptional(assignedTo)).build())
				.build();

		return handleGetAllRequest(requestCriteria);
	}

	@GetMapping(params = { PARAM_PAGE, PARAM_LIMIT, PARAM_NEXT_WIEDERVORLAGE_FRIST })
	public RepresentationModel<EntityModel<EnhancedVorgang>> getAllByHasNextWiedervorlageFrist(@RequestParam int page, @RequestParam Integer limit,
			@RequestParam String nextFrist) {
		var requestCriteria = FindVorgaengeHeaderRequestCriteria.builder().page(page).limit(limit)
				.filterCriteria(FilterCriteria.builder()
						.hasNextWiedervorlageFrist(existNextWiedervorlageFrist(nextFrist)).build())
				.build();

		return handleGetAllRequest(requestCriteria);
	}

	@GetMapping(params = { PARAM_PAGE, PARAM_LIMIT, PARAM_ASSIGNED_TO, PARAM_NEXT_WIEDERVORLAGE_FRIST })
	public RepresentationModel<EntityModel<EnhancedVorgang>> getAllByAssignedToAndHasNextWiedervorlageFrist(@RequestParam int page,
			@RequestParam Integer limit,
			@RequestParam UserId assignedTo, @RequestParam String nextFrist) {
		var requestCriteria = FindVorgaengeHeaderRequestCriteria.builder().page(page).limit(limit)
				.filterCriteria(FilterCriteria.builder()
						.filterByAssignedTo(true).assignedTo(asOptional(assignedTo))
						.hasNextWiedervorlageFrist(existNextWiedervorlageFrist(nextFrist))
						.build())
				.build();

		return handleGetAllRequest(requestCriteria);
	}

	boolean existNextWiedervorlageFrist(String nextWiedervorlageFrist) {
		return StringUtils.equals(nextWiedervorlageFrist, PARAM_NEXT_WIEDERVORLAGE_FRIST_EXISTS);
	}

	Optional<UserId> asOptional(UserId userId) {
		return StringUtils.isEmpty(userId.toString()) ? Optional.empty() : Optional.of(userId);
	}

	RepresentationModel<EntityModel<EnhancedVorgang>> handleGetAllRequest(FindVorgaengeHeaderRequestCriteria requestCriteria) {
		var vorgaengeHeaderResponse = vorgangService.findVorgaengeHeader(requestCriteria);

		return modelAssembler.toCollectionModel(vorgaengeHeaderResponse.getVorgaengeHeader().stream(), vorgaengeHeaderResponse, requestCriteria,
				statisticController.getVorgaengeStatistic());
	}

	@PostAuthorize(VorgangAuthorizationService.VERIFY_BY_USER_ROLE)
	@GetMapping("/{vorgangId}")
	public EntityModel<Vorgang> getVorgangWithEingang(@PathVariable String vorgangId) {
		return modelAssembler.toModel(getVorgang(vorgangId));
	}

	public VorgangWithEingang getVorgang(String vorgangId) {
		return vorgangService.findVorgangWithEingang(vorgangId);
	}

	@PutMapping(path = "/{vorgangId}/hasNewPostfachNachricht", consumes = MediaType.APPLICATION_JSON_VALUE)
	public void resetNewPostfachNachricht(@PathVariable String vorgangId, @RequestBody ResetNewPostfachNachrichtBody body,
			HttpServletResponse response) {
		if (body.isHasNewPostfachNachricht()) {
			response.setStatus(HttpStatus.UNPROCESSABLE_ENTITY.value());
			return;
		}

		clientAttributeService.resetPostfachNachricht(vorgangId);
		response.setStatus(HttpStatus.NO_CONTENT.value());
	}

	@GetMapping(params = { PARAM_PAGE, PARAM_LIMIT, PARAM_NACHRICHTEN })
	public RepresentationModel<EntityModel<EnhancedVorgang>> getAllByUngeleseneNachrichten(@RequestParam int page, @RequestParam Integer limit,
			@RequestParam String nachrichten) {
		var requestCriteria = FindVorgaengeHeaderRequestCriteria.builder().page(page).limit(limit)
				.filterCriteria(FilterCriteria.builder().hasUngeleseneNachrichten(PARAM_NACHRICHTEN_UNGELESENE.equals(nachrichten)).build())
				.build();

		return handleGetAllRequest(requestCriteria);
	}

	@GetMapping(params = { PARAM_PAGE, PARAM_LIMIT, PARAM_ASSIGNED_TO, PARAM_NACHRICHTEN })
	public RepresentationModel<EntityModel<EnhancedVorgang>> getAllByAssignedToAndUngeleseneNachrichten(@RequestParam int page,
			@RequestParam Integer limit, @RequestParam UserId assignedTo, @RequestParam String nachrichten) {
		var requestCriteria = FindVorgaengeHeaderRequestCriteria.builder().page(page).limit(limit)
				.filterCriteria(FilterCriteria.builder()
						.filterByAssignedTo(true).assignedTo(asOptional(assignedTo))
						.hasUngeleseneNachrichten(PARAM_NACHRICHTEN_UNGELESENE.equals(nachrichten))
						.build())
				.build();

		return handleGetAllRequest(requestCriteria);
	}
}