package de.ozgcloud.alfa.common.attacheditem;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.errorhandling.ResourceNotFoundException;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcFindVorgangAttachedItemRequest;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcFindVorgangAttachedItemResponse;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItem;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItemResponse;
import de.ozgcloud.vorgang.vorgangAttachedItem.VorgangAttachedItemServiceGrpc.VorgangAttachedItemServiceBlockingStub;
import io.grpc.Status;
import io.grpc.Status.Code;
import io.grpc.StatusRuntimeException;

class VorgangAttachedItemRemoteServiceTest {

	@Spy
	@InjectMocks
	private VorgangAttachedItemRemoteService service;

	@Mock
	private VorgangAttachedItemServiceBlockingStub grpcService;

	@Mock
	private VorgangAttachedItemMapper mapper;

	@Nested
	@DisplayName("FindVorgangAttachedItem")
	class TestFindVorgangAttachedItem {

		private final GrpcFindVorgangAttachedItemRequest request = GrpcFindVorgangAttachedItemRequestTestFactory.create();
		private final GrpcFindVorgangAttachedItemResponse response = GrpcFindVorgangAttachedItemResponseTestFactory.create();
		private final VorgangAttachedItem vorgangAttachedItem = VorgangAttachedItemTestFactory.create();

		@Captor
		private ArgumentCaptor<GrpcVorgangAttachedItem> grpcVorgangAttachedItemArgumentCaptor;

		@BeforeEach
		void init() {
			when(grpcService.find(request)).thenReturn(response);
			doReturn(request).when(service).buildFindRequest(VorgangHeaderTestFactory.ID, VorgangAttachedItemTestFactory.ITEM_NAME);
		}

		@Test
		void shouldCallGrpcService() {
			service.findVorgangAttachedItem(VorgangHeaderTestFactory.ID, VorgangAttachedItemTestFactory.ITEM_NAME);

			verify(grpcService).find(request);
		}

		@Test
		void shouldBuildRequest() {
			service.findVorgangAttachedItem(VorgangHeaderTestFactory.ID, VorgangAttachedItemTestFactory.ITEM_NAME);

			verify(service).buildFindRequest(VorgangHeaderTestFactory.ID, VorgangAttachedItemTestFactory.ITEM_NAME);
		}

		@Test
		void shouldMapGrpcResponse() {
			service.findVorgangAttachedItem(VorgangHeaderTestFactory.ID, VorgangAttachedItemTestFactory.ITEM_NAME).toList();

			verify(mapper).fromGrpc(grpcVorgangAttachedItemArgumentCaptor.capture());
			assertThat(grpcVorgangAttachedItemArgumentCaptor.getAllValues())
					.containsExactly(GrpcFindVorgangAttachedItemResponseTestFactory.VORGANG_ATTACHED_ITEM);
		}

		@Test
		void shouldReturnResult() {
			when(mapper.fromGrpc(any())).thenReturn(vorgangAttachedItem);

			var result = service.findVorgangAttachedItem(VorgangHeaderTestFactory.ID, VorgangAttachedItemTestFactory.ITEM_NAME);

			assertThat(result).hasSize(1).containsExactly(vorgangAttachedItem);
		}
	}

	@Nested
	@DisplayName("Build find vorgang attached item gRPC request")
	class TestBuildFindRequest {

		@Test
		void shouldBuild() {
			var request = service.buildFindRequest(VorgangHeaderTestFactory.ID, VorgangAttachedItemTestFactory.ITEM_NAME);

			assertThat(request.getItemName()).isEqualTo(VorgangAttachedItemTestFactory.ITEM_NAME);
			assertThat(request.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}
	}

	@Nested
	@DisplayName("Build get by id gRPC request")
	class TestBuildGetByIdRequest {

		@Test
		void shouldBuild() {
			var request = service.buildGetByIdRequest(VorgangAttachedItemTestFactory.ID);

			assertThat(request.getId()).isEqualTo(VorgangAttachedItemTestFactory.ID);
		}

	}

	@DisplayName("Get by id")
	@Nested
	class TestGetById {

		private final GrpcVorgangAttachedItemResponse response = GrpcVorgangAttachedItemResponseTestFactory.create();
		private final VorgangAttachedItem vorgangAttachedItem = VorgangAttachedItemTestFactory.create();

		@BeforeEach
		void init() {
			when(mapper.fromGrpc(any())).thenReturn(vorgangAttachedItem);
			doReturn(response).when(service).doGetById(any());
		}

		@Test
		void shouldDoGetById() {
			service.getById(VorgangAttachedItemTestFactory.ID);

			verify(service).doGetById(VorgangAttachedItemTestFactory.ID);
		}

		@Test
		void shouldMapResponse() {
			service.getById(VorgangAttachedItemTestFactory.ID);

			verify(mapper).fromGrpc(response.getVorgangAttachedItem());
		}

		@Test
		void shouldReturnResult() {
			var result = service.getById(VorgangAttachedItemTestFactory.ID);

			assertThat(result).isEqualTo(vorgangAttachedItem);
		}
	}

	@DisplayName("Do get by id")
	@Nested
	class TestDoGetById {

		@Test
		void shouldBuildRequest() {
			service.doGetById(VorgangAttachedItemTestFactory.ID);

			verify(service).buildGetByIdRequest(VorgangAttachedItemTestFactory.ID);
		}

		@Test
		void shouldCallGrpcService() {
			var request = GrpcVorgangAttachedItemRequestTestFactory.create();

			service.doGetById(VorgangAttachedItemTestFactory.ID);

			verify(grpcService).getById(request);
		}

		@DisplayName("on grpc exception")
		@Nested
		class TestOnGrpcException {

			@DisplayName("not found")
			@Nested
			class TestNotFound {

				@Test
				void shouldThrowResourceNotFoundException() {
					var statusRuntimeException = new StatusRuntimeException(Status.NOT_FOUND);

					when(grpcService.getById(any())).thenThrow(statusRuntimeException);

					assertThatThrownBy(() -> service.doGetById(VorgangAttachedItemTestFactory.ID)).isInstanceOf(ResourceNotFoundException.class);
				}
			}

			@ParameterizedTest
			@EnumSource(mode = Mode.EXCLUDE, names = { "NOT_FOUND" })
			void shouldThrowException(Code code) {
				var statusRuntimeException = new StatusRuntimeException(code.toStatus());

				when(grpcService.getById(any())).thenThrow(statusRuntimeException);

				assertThatThrownBy(() -> service.doGetById(VorgangAttachedItemTestFactory.ID)).isInstanceOf(StatusRuntimeException.class);
			}
		}
	}
}