/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.postfach;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.stream.Stream;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.core.io.Resource;
import org.springframework.test.util.ReflectionTestUtils;

import de.ozgcloud.alfa.common.GermanDateTimeFormatter;
import de.ozgcloud.alfa.common.binaryfile.BinaryFileTestFactory;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.postfach.PostfachMail.Direction;
import de.ozgcloud.alfa.postfach.PostfachNachrichtPdfModel.Nachricht;
import de.ozgcloud.alfa.vorgang.AntragstellerTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.common.pdf.PdfService;
import lombok.SneakyThrows;

class PostfachNachrichtPdfServiceTest {

	@Spy
	@InjectMocks
	private PostfachNachrichtPdfService service;
	@Mock
	private PdfService pdfService;
	@Mock
	private GermanDateTimeFormatter germanDateTimeFormatter;

	@DisplayName("Get all as pdf")
	@Nested
	class TestGetAllAsPdf {

		@Mock
		private OutputStream output;

		@DisplayName("on getting template")
		@Nested
		class TestGetTemplate {

			@Mock
			private Resource pdfTemplate;

			@BeforeEach
			void mockPdfTemplate() {
				ReflectionTestUtils.setField(service, "pdfTemplate", pdfTemplate);
			}

			@SneakyThrows
			@Test
			void shouldGetInputStreamFromResource() {
				when(pdfTemplate.getInputStream()).thenReturn(InputStream.nullInputStream());

				service.getTemplate();

				verify(pdfTemplate).getInputStream();
			}

			@SneakyThrows
			@Test
			void shouldReturnIfExists() {
				var inputStream = InputStream.nullInputStream();
				when(pdfTemplate.getInputStream()).thenReturn(inputStream);

				var templateInputStream = service.getTemplate();

				assertThat(templateInputStream).isEqualTo(inputStream);
			}

			@SneakyThrows
			@Test
			void shouldThrowExceptionIfMissing() {
				when(pdfTemplate.getInputStream()).thenThrow(IOException.class);

				assertThatThrownBy(() -> service.getTemplate()).isInstanceOf(TechnicalException.class);
			}
		}

		@DisplayName("build model")
		@Nested
		class TestBuildModel {

			@DisplayName("by vorgang")
			@Nested
			class TestByVorgang {

				@Test
				void shouldSetVorgangNummer() {
					var model = service.buildModel(VorgangWithEingangTestFactory.create(), Stream.empty());

					assertThat(model.getVorgangNummer()).isEqualTo(VorgangHeaderTestFactory.NUMMER);
				}

				@Test
				void shouldSetVorgangName() {
					var model = service.buildModel(VorgangWithEingangTestFactory.create(), Stream.empty());

					assertThat(model.getVorgangName()).isEqualTo(VorgangHeaderTestFactory.NAME);
				}

				@Nested
				class TestSetIsFirst {

					@Test
					void shouldSetIsFirstTrueOnFirstMessage() {
						var model = service.buildModel(VorgangWithEingangTestFactory.create(),
								Stream.of(PostfachNachrichtPdfDataTestFactory.create()));

						assertThat(model.getNachrichten().get(0).isFirst()).isTrue();
					}

					@Test
					void shouldSetIsFirstFalseOnOtherMessage() {
						var model = service.buildModel(VorgangWithEingangTestFactory.create(),
								Stream.of(PostfachNachrichtPdfDataTestFactory.create(), PostfachNachrichtPdfDataTestFactory.create(),
										PostfachNachrichtPdfDataTestFactory.create()));

						assertThat(model.getNachrichten().get(1).isFirst()).isFalse();
						assertThat(model.getNachrichten().get(2).isFirst()).isFalse();
					}
				}
			}

			@DisplayName("by Antragsteller")
			@Nested
			class TestMapAntragsteller {

				@Test
				void shouldMapAntragstellerAnrede() {
					var modelBuilder = mapAntragsteller();

					assertThat(modelBuilder.build().getAntragstellerAnrede()).isEqualTo(AntragstellerTestFactory.ANREDE);
				}

				@Test
				void shouldMapAntragstellerVorname() {
					var modelBuilder = mapAntragsteller();

					assertThat(modelBuilder.build().getAntragstellerVorname()).isEqualTo(AntragstellerTestFactory.VORNAME);
				}

				@Test
				void shouldMapAntragstellerNachname() {
					var modelBuilder = mapAntragsteller();

					assertThat(modelBuilder.build().getAntragstellerNachname()).isEqualTo(AntragstellerTestFactory.NACHNAME);
				}

				@Test
				void shouldMapAntragstellerStrasse() {
					var modelBuilder = mapAntragsteller();

					assertThat(modelBuilder.build().getAntragstellerStrasse()).isEqualTo(AntragstellerTestFactory.STRASSE);
				}

				@Test
				void shouldMapAntragstellerHausnummer() {
					var modelBuilder = mapAntragsteller();

					assertThat(modelBuilder.build().getAntragstellerHausnummer()).isEqualTo(AntragstellerTestFactory.HAUSNUMMER);
				}

				@Test
				void shouldMapAntragstellerPlz() {
					var modelBuilder = mapAntragsteller();

					assertThat(modelBuilder.build().getAntragstellerPlz()).isEqualTo(AntragstellerTestFactory.PLZ);
				}

				@Test
				void shouldMapAntragstellerOrt() {
					var modelBuilder = mapAntragsteller();

					assertThat(modelBuilder.build().getAntragstellerOrt()).isEqualTo(AntragstellerTestFactory.ORT);
				}

				private PostfachNachrichtPdfModel.PostfachNachrichtPdfModelBuilder mapAntragsteller() {
					var modelBuilder = PostfachNachrichtPdfModel.builder();

					service.mapAntragsteller(modelBuilder, AntragstellerTestFactory.create());

					return modelBuilder;
				}
			}

			@DisplayName("by postfachnachricht pdf data")
			@Nested
			class TestMapPostfachNachricht {

				@Test
				void shouldMapNachrichtSubject() {
					var nachricht = mapNachricht();

					assertThat(nachricht.getSubject()).isEqualTo(PostfachMailTestFactory.SUBJECT);
				}

				@Test
				void shouldMapNachrichtMailBody() {
					var nachricht = mapNachricht();

					assertThat(nachricht.getMailBody()).isEqualTo(PostfachMailTestFactory.MAIL_BODY);
				}

				@Test
				void shouldMapNachrichtCreatedAt() {
					var expected = "formatted date";
					when(germanDateTimeFormatter.formatZonedDateTime(PostfachMailTestFactory.CREATED_AT)).thenReturn(expected);

					var nachricht = mapNachricht();

					assertThat(nachricht.getCreatedAt()).isEqualTo(expected);
				}

				@Test
				void shouldCallDateFormatter() {
					mapNachricht();

					verify(germanDateTimeFormatter).formatZonedDateTime(PostfachMailTestFactory.CREATED_AT);
				}

				@Test
				void shouldMapNachrichtCreatedBy() {
					var nachricht = mapNachricht();

					assertThat(nachricht.getCreatedBy()).isEqualTo(UserProfileTestFactory.FULLNAME);
				}

				@Test
				void shouldMapNachrichtAttachments() {
					var nachricht = mapNachricht();

					assertThat(nachricht.getAttachments()).containsExactly(BinaryFileTestFactory.NAME);
				}

				private Nachricht mapNachricht() {
					return mapNachricht(PostfachNachrichtPdfDataTestFactory.create());
				}

				private PostfachNachrichtPdfModel.Nachricht mapNachricht(PostfachNachrichtPdfData pdfData) {
					return service.mapPostfachNachricht(pdfData, AntragstellerTestFactory.create());
				}

				@DisplayName("for incoming nachricht")
				@Nested
				class TestDirectionIn {

					@DisplayName("build absender name")
					@Nested
					class TestBuildUserName {

						private final PostfachNachrichtPdfData data = PostfachNachrichtPdfDataTestFactory.createBuilder().direction(Direction.IN)
								.build();

						@Test
						void shouldReturnAntragstellerIfExists() {
							var name = service.buildAbsenderName(data, AntragstellerTestFactory.create());

							assertThat(name).isEqualTo(AntragstellerTestFactory.VORNAME + " " + AntragstellerTestFactory.NACHNAME);
						}

						@Test
						void shouldReturnFallbackNameForAntragsteller() {
							var name = service.buildAbsenderName(data, null);

							assertThat(name).isEqualTo(PostfachNachrichtPdfService.FALLBACK_ANTRAGSTELLER_NAME);
						}

						@Test
						void shouldReturnEmptyStringOnNullAntragstellerName() {
							var name = service.buildAbsenderName(data, AntragstellerTestFactory.createBuilder().vorname(null).nachname(null).build());

							assertThat(name).isEqualTo(StringUtils.EMPTY);
						}
					}
				}

				@DisplayName("for outgoing nachricht")
				@Nested
				class TestDirectionOut {

					@DisplayName("build absender name")
					@Nested
					class TestBuildUserName {

						private final PostfachNachrichtPdfData data = PostfachNachrichtPdfDataTestFactory.createBuilder().direction(Direction.OUT)
								.build();

						@Test
						void shouldReturnUserProfileNameIfExists() {
							var name = service.buildAbsenderName(data, AntragstellerTestFactory.create());

							assertThat(name).isEqualTo(UserProfileTestFactory.FIRSTNAME + " " + UserProfileTestFactory.LASTNAME);
						}

						@Test
						void shouldReturnFallbackNameForUserProfile() {
							var name = service.buildAbsenderName(
									PostfachNachrichtPdfDataTestFactory.createBuilder().direction(Direction.OUT).user(null).build(),
									AntragstellerTestFactory.create());

							assertThat(name).isEqualTo(PostfachNachrichtPdfService.FALLBACK_USER_NAME);
						}

						@Test
						void shouldReturnEmptyStringOnNullName() {
							var data = PostfachNachrichtPdfDataTestFactory.createBuilder().direction(Direction.OUT)
									.user(UserProfileTestFactory.createBuilder().firstName(null).lastName(null).build()).build();

							var name = service.buildAbsenderName(data, AntragstellerTestFactory.create());

							assertThat(name).isEmpty();
						}
					}
				}
			}
		}

		@Test
		void shouldCallPdfService() {
			doReturn(InputStream.nullInputStream()).when(service).getTemplate();

			service.getAllAsPdf(VorgangWithEingangTestFactory.create(), Stream.of(PostfachNachrichtPdfDataTestFactory.create()), output);

			verify(pdfService).createPdf(any(InputStream.class), eq(output), any(PostfachNachrichtPdfModel.class));
		}
	}
}