/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang.forwarding;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.InputStream;
import java.util.Optional;
import java.util.Scanner;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.Spy;

import de.ozgcloud.common.test.TestUtils;

class LandesnetzInfoReadServiceTest {

	@Spy
	private LandesnetzInfoReadService service = new LandesnetzInfoReadService();

	@Nested
	class TestStartOfSection {

		@Test
		void shouldAcceptStartSectionLine() {
			boolean isStartOfSection = service.isStartOfSection("############################## Beginn Schleswig-Holstein ##########################");

			assertThat(isStartOfSection).isTrue();
		}

		@DisplayName("should return FALSE when")
		@ParameterizedTest(name = "\"{0}\"")
		@ValueSource(strings = {"", "0", "###", "abs.de 	esm", "################## Schleswig-Holstein ########"})
		void shouldDeclineAnotherLines(String line) {
			boolean isStartOfSection = service.isStartOfSection(line);

			assertThat(isStartOfSection).isFalse();
		}
	}

	@Nested
	class TestEndOfSection {

		@Test
		void shouldAcceptStartSectionLine() {
			boolean isEndOfSection = service.isEndOfSection("############################## Ende Schleswig-Holstein ##########################");

			assertThat(isEndOfSection).isTrue();
		}

		@DisplayName("should return FALSE when")
		@ParameterizedTest(name = "\"{0}\"")
		@ValueSource(strings = {"", "0", "###", "abs.de 	esm","################## Schleswig-Holstein ########"})
		void shouldDeclineAnotherLines(String line) {
			boolean isEndOfSection = service.isEndOfSection(line);

			assertThat(isEndOfSection).isFalse();
		}
	}

	@Nested
	class TestIsComment {

		@Test
		void shouldAcceptCommentLine() {
			boolean isCommentedLine = service.isCommentedLine("# Kommentar");

			assertThat(isCommentedLine).isTrue();
		}

		@DisplayName("should return FALSE when")
		@ParameterizedTest(name = "\"{0}\"")
		@ValueSource(strings = {"", "0", "abs.de 	esm"})
		void shouldDeclineAnotherLines(String line) {
			boolean isCommentedLine = service.isCommentedLine(line);

			assertThat(isCommentedLine).isFalse();
		}
	}

	@Nested
	class TestGetFirsColumn {

		@Test
		void shouldReturnFirstColumn() {
			var firstColumn = service.getFirstColumn("abs.de 	esm");

			assertThat(firstColumn).get().isEqualTo("abs.de");
		}

		@Test
		void shouldReturnEmptyOptionalIfLineIsEmpty() {
			var firstColumn = service.getFirstColumn("");

			assertThat(firstColumn).isEmpty();
		}

	}

	@Nested
	class TestReadSection {

		@Test
		void shouldReturnEmptyResult() {
			var section = service.readSection(new Scanner(""));

			assertThat(section).isEmpty();
		}

		@Test
		void shouldCallIsCommentedLine() {
			var row = "# comment";

			service.readSection(new Scanner(row));

			verify(service).isCommentedLine(row);
		}

		@Test
		void shouldCallIsEndOfSection() {
			var line = "row";

			service.readSection(new Scanner(line));

			verify(service).isEndOfSection(line);
		}

		@Test
		void shouldCallGetFirstColumn() {
			var line = "row";

			service.readSection(new Scanner(line));

			verify(service).getFirstColumn(line);
		}

		@Test
		void shouldCollectFirstColumn() {
			var line = "row";
			doReturn(Optional.of(line)).when(service).getFirstColumn(anyString());

			var section = service.readSection(new Scanner(line+ "\n# Ende Schleswig-Holstein"));

			assertThat(section).containsExactly(line);
		}
	}

	@Nested
	class TestParseFile {

		private InputStream TEXT_INPUT_STREAM;

		@BeforeEach
		void initResource() {
			TEXT_INPUT_STREAM = TestUtils.loadFile("files/mailertable.txt");
		}

		@Test
		void shouldReadFileIfExist() {
			var parsedDomainList = service.readFile(TEXT_INPUT_STREAM);

			assertThat(parsedDomainList).hasSize(3);
		}

		@Test
		void shouldReturnEmptyListIfFileNotExist() {
			var parsedDomainList = service.readFile(InputStream.nullInputStream());

			assertThat(parsedDomainList).isEmpty();
		}

	}
}
