package de.ozgcloud.alfa.postfach;

import java.time.ZonedDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import de.ozgcloud.alfa.common.AnlageDokumentTypeBuilder;
import de.ozgcloud.alfa.common.DateConverter;
import de.ozgcloud.alfa.common.IdentifikationObjektTypeBuilder;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.user.UserProfile;
import de.ozgcloud.alfa.postfach.PostfachMail.Direction;
import de.xoev.xdomea.AnlageDokumentType;
import de.xoev.xdomea.DokumentType;
import de.xoev.xdomea.HistorienProtokollInformationType;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
class DokumentTypeBuilder {
	private static final String TYP = "Nachricht";
	private static final String ANTRAGSTELLER = "Antragsteller";
	private final Map<Direction, String> mapDirectionToString = Map.of(
			Direction.IN, "Nachricht empfangen",
			Direction.OUT, "Nachricht gesendet");

	private PostfachMail postfachMail;
	private List<OzgFile> ozgFileAttachments;
	private UserProfile userProfile;
	private String organisationseinheitenId;

	public static DokumentTypeBuilder builder() {
		return new DokumentTypeBuilder();
	}

	public DokumentTypeBuilder withPostfachMail(PostfachMail postfachMail) {
		this.postfachMail = postfachMail;
		return this;
	}

	public DokumentTypeBuilder withOzgFiles(List<OzgFile> ozgFiles) {
		this.ozgFileAttachments = ozgFiles;
		return this;
	}

	public DokumentTypeBuilder withUserProfile(UserProfile userProfile) {
		this.userProfile = userProfile;
		return this;
	}

	public DokumentTypeBuilder withOrganisationseinheitenId(String organisationseinheitenId) {
		this.organisationseinheitenId = organisationseinheitenId;
		return this;
	}

	public DokumentType build() {
		var dokumentType = new DokumentType();
		dokumentType.setIdentifikation(IdentifikationObjektTypeBuilder.builder()
				.withObjectID(postfachMail.getId())
				.build());
		dokumentType.setTyp(TYP);
		dokumentType.getHistorienProtokollInformation().add(createHistorienProtokollInformation());
		Optional.ofNullable(ozgFileAttachments).orElseGet(Collections::emptyList).stream().map(this::createAnlage)
				.forEach(dokumentType.getAnlage()::add);
		return dokumentType;
	}

	AnlageDokumentType createAnlage(OzgFile ozgFile) {
		return AnlageDokumentTypeBuilder.builder().withOzgFile(ozgFile).build();
	}

	HistorienProtokollInformationType createHistorienProtokollInformation() {
		var historienInformation = new HistorienProtokollInformationType();
		historienInformation.setMetadatumName(postfachMail.getMailBody());
		historienInformation.setAkteur(getAkteur());
		historienInformation.setDatumUhrzeit(DateConverter.toXmlGregorianCalendar(getSentTime()));
		historienInformation.setAktion(mapDirectionToString.get(postfachMail.getDirection()));
		return historienInformation;
	}

	String getAkteur() {
		return Optional.ofNullable(userProfile)
				.map(user -> user.getFullName() + "; " + organisationseinheitenId)
				.orElse(ANTRAGSTELLER);
	}

	ZonedDateTime getSentTime() {
		return postfachMail.getDirection() == Direction.IN ? postfachMail.getCreatedAt() : postfachMail.getSentAt();
	}

}
