package de.ozgcloud.alfa.export;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullAndEmptySource;
import org.mockito.Spy;
import org.springframework.validation.Errors;

class XdomeaPropertiesValidatorTest {

	@Spy
	private XdomeaPropertiesValidator validator;

	@Nested
	class TestSupports {

		@Test
		void shouldSupportXdomeaProperties() {
			var supports = validator.supports(XdomeaProperties.class);

			assertThat(supports).isTrue();
		}
	}

	@Nested
	class TestValidate {

		@Test
		void shouldBeValidIfAllBehoerdenschluesselPropertiesAreSet() {
			var properties = XdomeaPropertiesTestFactory.create();

			var errors = validate(properties);

			assertThat(errors.hasErrors()).isFalse();
		}

		@ParameterizedTest
		@NullAndEmptySource
		void shouldBeValidIfBehoerdenschluesselIsNotSet(String behoerdenschluessel) {
			var properties = XdomeaPropertiesTestFactory.createBuilder().behoerdenschluessel(behoerdenschluessel).build();

			var errors = validate(properties);

			assertThat(errors.hasErrors()).isFalse();
		}

		@Nested
		class OnUriNotSet {

			private static final String PROPERTY_NAME = "behoerdenschluesselUri";
			public static final String PROPERTY_PATH = "ozgcloud.xdomea." + PROPERTY_NAME;
			public static final String ERROR_CODE = PROPERTY_PATH + ".empty";

			@ParameterizedTest
			@NullAndEmptySource
			void shouldHaveFieldErrors(String uri) {
				var properties = withBehoerdenschluesselUri(uri);

				var errors = validate(properties);

				assertThat(errors.hasFieldErrors(PROPERTY_NAME)).isTrue();
			}

			@ParameterizedTest
			@NullAndEmptySource
			void shouldHaveMeaningfulErrorMessage(String uri) {
				var properties = withBehoerdenschluesselUri(uri);

				var errors = validate(properties);

				assertThat(errors.getFieldError(PROPERTY_NAME).getDefaultMessage()).contains(PROPERTY_NAME);
			}

			@ParameterizedTest
			@NullAndEmptySource
			void shouldHaveErrorCode(String uri) {
				var properties = withBehoerdenschluesselUri(uri);

				var errors = validate(properties);

				assertThat(errors.getFieldError(PROPERTY_NAME).getCode()).isEqualTo(ERROR_CODE);
			}

			private XdomeaProperties withBehoerdenschluesselUri(String uri) {
				return XdomeaPropertiesTestFactory.createBuilder().behoerdenschluesselUri(uri).build();
			}
		}

		@Nested
		class OnVersionNotSet {

			public static final String PROPERTY_NAME = "behoerdenschluesselVersion";
			public static final String PROPERTY_PATH = "ozgcloud.xdomea." + PROPERTY_NAME;
			public static final String ERROR_CODE = PROPERTY_PATH + ".empty";

			@ParameterizedTest
			@NullAndEmptySource
			void shouldHaveFieldErrors(String version) {
				var properties = withBehoerdenschluesselVersion(version);

				var errors = validate(properties);

				assertThat(errors.hasFieldErrors(PROPERTY_NAME)).isTrue();
			}

			@ParameterizedTest
			@NullAndEmptySource
			void shouldHaveMeaningfulErrorMessage(String version) {
				var properties = withBehoerdenschluesselVersion(version);

				var errors = validate(properties);

				assertThat(errors.getFieldError(PROPERTY_NAME).getDefaultMessage()).contains(PROPERTY_NAME);
			}

			@ParameterizedTest
			@NullAndEmptySource
			void shouldHaveErrorCode(String version) {
				var properties = withBehoerdenschluesselVersion(version);

				var errors = validate(properties);

				assertThat(errors.getFieldError(PROPERTY_NAME).getCode()).isEqualTo(ERROR_CODE);
			}

			private XdomeaProperties withBehoerdenschluesselVersion(String version) {
				return XdomeaPropertiesTestFactory.createBuilder().behoerdenschluesselVersion(version).build();
			}
		}

		private Errors validate(XdomeaProperties properties) {
			return validator.validateObject(properties);
		}
	}
}
